import uuid
from typing import Callable
from unittest.mock import MagicMock

import pytest
import requests_mock

from endstone_bstats import MetricsBase
from endstone_bstats._executor import ScheduledThreadPoolExecutor


@pytest.fixture
def metrics(mocker):
    mocker.patch.object(ScheduledThreadPoolExecutor, "submit")
    mocker.patch.object(ScheduledThreadPoolExecutor, "submit_at_fixed_rate")
    m = MetricsBase(
        platform="test_platform",
        server_uuid=uuid.uuid4(),
        service_id=1,
        enabled=True,
        platform_data_appender=mocker.MagicMock(),
        service_data_appender=mocker.MagicMock(),
        task_submitter=None,
        check_service_enabled=mocker.MagicMock(return_value=True),
        error_logger=mocker.MagicMock(),
        info_logger=mocker.MagicMock(),
        log_errors=True,
        log_sent_data=True,
        log_response_status_text=True,
    )
    yield m
    m.shutdown()


def test_add_custom_chart(mocker, metrics):
    chart = mocker.MagicMock()
    metrics.add_custom_chart(chart)
    assert chart in metrics._custom_charts


def test_shutdown(mocker, metrics):
    mocker.patch.object(
        ScheduledThreadPoolExecutor, "shutdown", wraps=metrics._executor.shutdown
    )
    metrics.shutdown()
    metrics._executor.shutdown.assert_called_once()


def test_submit_data(mocker, metrics):
    mocker.patch.object(MetricsBase, "_send_data")
    chart = mocker.MagicMock()
    metrics.add_custom_chart(chart)
    metrics._submit_data()
    data = metrics._send_data.call_args.args[0]
    assert data["service"]["id"] == 1


def test_submit_data_with_exception(mocker, metrics):
    mocker.patch.object(
        MetricsBase, "_send_data", side_effect=Exception("Test exception")
    )
    chart = mocker.MagicMock()
    metrics.add_custom_chart(chart)
    metrics._submit_data()
    metrics._error_logger.assert_called_once()


def test_send_data(mocker, metrics):
    with requests_mock.Mocker() as rm:
        rm.post("https://bStats.org/api/v2/data/test_platform", status_code=201)
        metrics._submit_data()
        assert rm.last_request.body is not None
        assert metrics._info_logger.call_count == 2


def warp_submit(task: Callable, *args, **kwargs):
    task()


def test_start_submitting(mocker, metrics):
    mocker.patch.object(ScheduledThreadPoolExecutor, "submit", wraps=warp_submit)
    metrics._start_submitting()


def test_start_submitting_with_task_submitter(mocker, metrics):
    mocker.patch.object(ScheduledThreadPoolExecutor, "submit", wraps=warp_submit)
    metrics._task_submitter = MagicMock()
    metrics._start_submitting()
    metrics._task_submitter.assert_called_once()


def test_start_submitting_when_disabled(mocker, metrics):
    mocker.patch.object(ScheduledThreadPoolExecutor, "submit", wraps=warp_submit)
    metrics._enabled = False
    mocker.patch.object(MetricsBase, "shutdown", wraps=metrics.shutdown)
    metrics._start_submitting()
    metrics.shutdown.assert_called_once()



import gzip
import json
import random
import uuid
from io import BytesIO
from typing import Any, Callable, Dict, Optional, Set

import requests

from endstone_bstats._executor import ScheduledThreadPoolExecutor


class MetricsBase:
    """
    The MetricsBase class to handle sending metrics to bStats.

    Attributes:
        METRICS_VERSION (str): The version of the Metrics class.
        REPORT_URL (str): The URL to which data is reported.
    """

    METRICS_VERSION = "3.0.3"
    REPORT_URL = "https://bStats.org/api/v2/data/{platform}"

    def __init__(
        self,
        platform: str,
        server_uuid: uuid.UUID,
        service_id: int,
        enabled: bool,
        platform_data_appender: Callable[[Dict[str, Any]], None],
        service_data_appender: Callable[[Dict[str, Any]], None],
        task_submitter: Optional[Callable[[Callable[[], None]], None]],
        check_service_enabled: Callable[[], bool],
        error_logger: Callable[[str, Exception], None],
        info_logger: Callable[[str], None],
        log_errors: bool,
        log_sent_data: bool,
        log_response_status_text: bool,
    ):
        """
        Initializes the MetricsBase instance.

        Args:
            platform (str): The platform of the service.
            server_uuid (uuid.UUID): The server UUID.
            service_id (int): The service ID.
            enabled (bool): Whether or not data sending is enabled.
            platform_data_appender (Callable[[Dict[str, Any]], None]): A consumer to append platform-specific data.
            service_data_appender (Callable[[Dict[str, Any]], None]): A consumer to append service-specific data.
            task_submitter (Optional[Callable[[Callable[[], None]], None]]): A consumer to handle the submit task.
            check_service_enabled (Callable[[], bool]): A supplier to check if the service is still enabled.
            error_logger (Callable[[str, Exception], None]): A consumer for error logging.
            info_logger (Callable[[str], None]): A consumer for info logging.
            log_errors (bool): Whether or not errors should be logged.
            log_sent_data (bool): Whether or not the sent data should be logged.
            log_response_status_text (bool): Whether or not the response status text should be logged.
        """
        self._platform = platform
        self._server_uuid = server_uuid
        self._service_id = service_id
        self._enabled = enabled
        self._platform_data_appender = platform_data_appender
        self._service_data_appender = service_data_appender
        self._task_submitter = task_submitter
        self._check_service_enabled = check_service_enabled
        self._error_logger = error_logger
        self._info_logger = info_logger
        self._log_errors = log_errors
        self._log_sent_data = log_sent_data
        self._log_response_status_text = log_response_status_text
        self._custom_charts: Set = set()
        self._executor = ScheduledThreadPoolExecutor(max_workers=1)

        if self._enabled:
            self._start_submitting()

    def add_custom_chart(self, chart: Any):
        """
        Adds a custom chart.

        Args:
            chart: The custom chart to add.
        """
        self._custom_charts.add(chart)

    def shutdown(self):
        """Shuts down the scheduler."""
        self._executor.shutdown()

    def _start_submitting(self):
        """
        Starts the submitting process with initial and periodic delays.
        """

        def submit_task():
            if not self._enabled or not self._check_service_enabled():
                self.shutdown()
                return

            if self._task_submitter is not None:
                self._task_submitter(self._submit_data)
            else:
                self._submit_data()

        initial_delay = int((3 + random.random() * 3) * 60)
        second_delay = int((random.random() * 30) * 60)

        self._executor.submit(submit_task, initial_delay)
        self._executor.submit_at_fixed_rate(
            submit_task, initial_delay + second_delay, 60 * 30
        )

    def _submit_data(self):
        """
        Constructs the JSON data and sends it to bStats.
        """

        platform_data = {}
        self._platform_data_appender(platform_data)

        service_data = {}
        self._service_data_appender(service_data)

        chart_data = []
        for chart in self._custom_charts:
            chart_data.append(
                chart.get_request_json_object(self._error_logger, self._log_errors)
            )

        service_data["id"] = self._service_id
        service_data["customCharts"] = chart_data
        platform_data["service"] = service_data
        platform_data["serverUUID"] = str(self._server_uuid)
        # platform_data["metricsVersion"] = self.METRICS_VERSION

        try:
            self._send_data(platform_data)
        except Exception as e:
            if self._log_errors:
                self._error_logger("Could not submit bStats metrics data", e)

    def _send_data(self, data: Dict[str, Any]):
        """
        Sends the JSON data to bStats.

        Args:
            data: The JSON data to send.
        """
        if self._log_sent_data:
            self._info_logger(f"Sent bStats metrics data: {data}")

        url = self.REPORT_URL.format(platform=self._platform)
        compressed_data = self._compress(data)

        headers = {
            "Accept": "application/json",
            "Connection": "close",
            "Content-Encoding": "gzip",
            "Content-Length": str(len(compressed_data)),
            "Content-Type": "application/json",
            "User-Agent": "Metrics-Service/1",
        }

        response = requests.post(url, headers=headers, data=compressed_data)
        response.raise_for_status()

        if self._log_response_status_text:
            self._info_logger(
                f"Sent data to bStats and received response: {response.text}"
            )

    @staticmethod
    def _compress(data: Dict[str, Any]) -> bytes:
        """
        Compresses the given data using gzip.

        Args:
            data (dict): The data to be compressed.

        Returns:
            bytes: The compressed data.
        """
        bio = BytesIO()
        with gzip.GzipFile(fileobj=bio, mode="wb") as gzip_file:
            gzip_file.write(json.dumps(data).encode("utf-8"))

        return bio.getvalue()



import pytest

from endstone_bstats import AdvancedPie


def test_valid_advanced_pie():
    chart = AdvancedPie("valid_advanced_pie", lambda: {"key1": 10, "key2": 5})
    assert chart.get_chart_data() == {"values": {"key1": 10, "key2": 5}}


def test_advanced_pie_chart_data_empty_dict():
    chart = AdvancedPie("advanced_pie_chart_with_empty_dict", lambda: {})
    assert chart.get_chart_data() is None


def test_advanced_pie_chart_data_none():
    chart = AdvancedPie("advanced_pie_chart_with_none", lambda: None)
    assert chart.get_chart_data() is None


def test_advanced_pie_chart_data_all_zero_values():
    chart = AdvancedPie(
        "advanced_pie_chart_with_all_zero_values", lambda: {"key1": 0, "key2": 0}
    )
    assert chart.get_chart_data() is None


def test_advanced_pie_chart_data_mixed_values():
    chart = AdvancedPie(
        "advanced_pie_chart_with_mixed_values",
        lambda: {"key1": 0, "key2": 10, "key3": 0},
    )
    assert chart.get_chart_data() == {"values": {"key2": 10}}


if __name__ == "__main__":
    pytest.main()



import os
import platform
from functools import partial
from pathlib import Path
from typing import Any, Dict

from endstone.plugin import Plugin

from endstone_bstats._base import MetricsBase
from endstone_bstats._charts.custom_chart import CustomChart
from endstone_bstats._config import MetricsConfig


class Metrics:
    def __init__(self, plugin: Plugin, service_id: int):
        """
        Creates a new Metrics instance.

        Args:
            plugin (Plugin): Your plugin instance.
            service_id (int): The id of the service.
                              It can be found at https://bstats.org/what-is-my-plugin-id
        """
        self._plugin = plugin

        # Get the config file
        bstats_folder = Path(plugin.data_folder).parent / "bstats"
        config_file = bstats_folder / "config.toml"
        config = MetricsConfig(config_file, True)

        self._metrics_base = MetricsBase(
            platform="server-implementation",
            server_uuid=config.server_uuid,
            service_id=service_id,
            enabled=config.enabled,
            platform_data_appender=self.append_platform_data,
            service_data_appender=self.append_service_data,
            task_submitter=lambda task: partial(
                plugin.server.scheduler.run_task, plugin, task
            ),
            check_service_enabled=lambda: plugin.enabled,
            error_logger=lambda msg, e: plugin.logger.warning(f"{msg}: {e}"),
            info_logger=plugin.logger.info,
            log_errors=config.log_errors_enabled,
            log_sent_data=config.log_sent_data_enabled,
            log_response_status_text=config.log_response_status_text_enabled,
        )

    def shutdown(self):
        """Shuts down the underlying scheduler service."""
        self._metrics_base.shutdown()

    def add_custom_chart(self, chart: CustomChart):
        """
        Adds a custom chart.

        Args:
            chart (CustomChart): The chart to add.
        """
        self._metrics_base.add_custom_chart(chart)

    def append_platform_data(self, platform_data: Dict[str, Any]) -> None:
        """
        Appends platform-specific data to the provided dict.

        Args:
            platform_data (Dict[str, Any]): The dict to append data to.
        """
        # TODO: implement the following
        platform_data["playerAmount"] = len(self._plugin.server.online_players)
        # platform_data["onlineMode"] = 1 if Bukkit.get_online_mode() else 0

        platform_data["osName"] = platform.system()
        platform_data["osArch"] = platform.machine().lower()
        platform_data["osVersion"] = platform.release()
        platform_data["coreCount"] = os.cpu_count()

    def append_service_data(self, service_data: Dict[str, Any]):
        """
        Appends service-specific data to the provided dict.

        Args:
            service_data (Dict[str, Any]): The dict to append data to.
        """
        service_data["pluginVersion"] = self._plugin.description.version



from __future__ import annotations

from typing import Callable

from endstone_bstats._charts.custom_chart import CustomChart


class SimplePie(CustomChart):
    def __init__(self, chart_id: str, get_value: Callable[[], str | None]) -> None:
        """
        Class constructor.

        Args:
            chart_id (str): The id of the chart.
            get_value (Callable[[], str | None]): The callable which is used to request the chart data.
        """

        super().__init__(chart_id)
        self.get_value = get_value

    def get_chart_data(self) -> dict | None:
        """
        Gets the data for the simple pie chart.

        Returns:
            dict | None: A dictionary with the chart data.
        """

        data = {}
        value = self.get_value()
        if not value:
            return None  # skip the chart

        data["value"] = value
        return data



from __future__ import annotations

from abc import ABC, abstractmethod

from endstone_bstats._errors import ChartDataError


class CustomChart(ABC):
    """
    Represents a custom chart.
    """

    def __init__(self, chart_id: str) -> None:
        """
        Class constructor.

        Args:
            chart_id (str): The id of the chart.

        Raises:
            ValueError: If chart_id is None or empty.
        """
        if not chart_id:
            raise ValueError("chart_id cannot be None or empty!")

        self.chart_id = chart_id

    def _get_request_json_object(self) -> dict | None:
        """
        Gets the JSON object for the chart request.

        Returns:
            dict: A dictionary representing the chart, or None if the data is invalid or an exception occurs.

        Raises:
            ChartDataError: If there is an error getting the chart data.
        """
        chart = {"chartId": self.chart_id}
        try:
            data = self.get_chart_data()
            if not data:  # If the data is empty we don't send the chart.
                return None

            chart["data"] = data

        except Exception as e:
            raise ChartDataError(
                f"Failed to get data for custom chart with id {self.chart_id}"
            ) from e

        return chart

    @abstractmethod
    def get_chart_data(self) -> dict | None:
        """
        Gets the data for the chart.

        Returns:
            dict: A dictionary with the chart data.

        Raises:
            Exception: If there is an error getting the chart data.
        """



import threading
import time
from concurrent.futures import ThreadPoolExecutor
from typing import Callable

import schedule


class ScheduledThreadPoolExecutor:
    def __init__(self, max_workers: int):
        self.scheduler = schedule.Scheduler()
        self.shutdown_event = threading.Event()
        self.executor = ThreadPoolExecutor(max_workers=max_workers)
        self.executor.submit(self.run)

    def run(self) -> None:
        while not self.shutdown_event.is_set():
            self.scheduler.run_pending()
            time.sleep(0.1)

    def shutdown(self) -> None:
        self.shutdown_event.set()
        self.executor.shutdown(wait=True, cancel_futures=True)
        self.scheduler.clear()

    def submit(self, task: Callable, delay_secs: int, *args, **kwargs) -> None:
        self.submit_at_fixed_rate(task, delay_secs, 0, *args, **kwargs)

    def submit_at_fixed_rate(
        self, task: Callable, initial_delay_secs: int, period_secs: int, *args, **kwargs
    ) -> None:
        def initial_run():
            task(*args, **kwargs)
            if period_secs > 0:
                self.scheduler.every(period_secs).seconds.do(task, *args, **kwargs)
            return schedule.CancelJob

        if initial_delay_secs > 0:
            self.scheduler.every(initial_delay_secs).seconds.do(initial_run)
        else:
            initial_run()



import uuid
from pathlib import Path

import tomlkit


class MetricsConfig:
    """
    A simple config for bStats.
    """

    def __init__(self, file: Path, default_enabled: bool) -> None:
        self._file = file
        self._default_enabled = default_enabled

        self._server_uuid: uuid.UUID | None = None
        self._enabled = False
        self._log_errors = False
        self._log_sent_data = False
        self._log_response_status_text = False
        self._did_exist_before = True

        self.setup_config()

    @property
    def server_uuid(self) -> uuid.UUID:
        return self._server_uuid

    @property
    def enabled(self) -> bool:
        return self._enabled

    @property
    def log_errors_enabled(self) -> bool:
        return self._log_errors

    @property
    def log_sent_data_enabled(self) -> bool:
        return self._log_sent_data

    @property
    def log_response_status_text_enabled(self) -> bool:
        return self._log_response_status_text

    @property
    def did_exist_before(self) -> bool:
        return self._did_exist_before

    def setup_config(self) -> None:
        """
        Creates the config file if it does not exist and read its content.
        """
        if not self._file.exists():
            self._did_exist_before = False
            self.save_default_config()

        self.load_config()

    def save_default_config(self) -> None:
        """
        Creates a config file with the default content.
        """
        comments = [
            "bStats (https://bStats.org) collects some basic information for plugin authors, like",
            "how many people use their plugin and their total player count. It's recommended to keep",
            "bStats enabled, but if you're not comfortable with this, you can turn this setting off.",
            "There is no performance penalty associated with having metrics enabled, and data sent to",
            "bStats is fully anonymous.",
        ]
        config = tomlkit.document()
        for comment in comments:
            config.add(tomlkit.comment(comment))

        config.add("enabled", self._default_enabled)
        config.add("server-uuid", str(uuid.uuid4()))
        config.add("log-errors", False)
        config.add("log-sent-data", False)
        config.add("log-response-status-text", False)

        self._file.parent.mkdir(parents=True, exist_ok=True)
        with open(self._file, "w", encoding="utf-8") as f:
            tomlkit.dump(config, f)

    def load_config(self) -> None:
        """
        Reads the content of the config file.
        """
        with open(self._file, "r", encoding="utf-8") as f:
            config = tomlkit.load(f)

        if "server-uuid" not in config:
            # Found a malformed config file with no UUID. Let's recreate it.
            self.save_default_config()
            self.load_config()
            return

        self._enabled = config.get("enabled", True)
        self._server_uuid = uuid.UUID(config.get("server-uuid"))
        self._log_errors = config.get("log-errors", False)
        self._log_sent_data = config.get("log-sent-data", False)
        self._log_response_status_text = config.get("log-response-status-text", False)



import uuid
from pathlib import Path

import pytest
import tomlkit

from endstone_bstats import MetricsConfig


@pytest.fixture
def temp_config_file(tmp_path: Path):
    """
    Fixture that creates a temporary config file.
    """
    return tmp_path / "config.toml"


def test_initialization_creates_file(temp_config_file):
    """
    Test whether the MetricsConfig properly creates a config file if it does not exist.
    """
    config = MetricsConfig(temp_config_file, default_enabled=True)

    # Check if the config file was created
    assert temp_config_file.exists()
    assert config.did_exist_before is False


def test_loading_existing_config(temp_config_file):
    """
    Test loading an existing config file.
    """

    # Manually create a config file
    config_content = tomlkit.document()
    config_content["enabled"] = True
    config_content["server-uuid"] = str(uuid.uuid4())
    config_content["log-errors"] = True
    config_content["log-sent-data"] = True
    config_content["log-response-status-text"] = True

    with open(temp_config_file, "w", encoding="utf-8") as f:
        tomlkit.dump(config_content, f)

    config = MetricsConfig(temp_config_file, default_enabled=False)

    assert config.enabled is True
    assert config.log_errors_enabled is True
    assert config.log_sent_data_enabled is True
    assert config.log_response_status_text_enabled is True
    assert config.did_exist_before is True


def test_malformed_config_creates_new_config(temp_config_file):
    """
    Test if it recreates the config file on malformed content.
    """

    # Create a malformed config file
    config_content = tomlkit.document()
    config_content["enabled"] = True

    with open(temp_config_file, "w", encoding="utf-8") as f:
        tomlkit.dump(config_content, f)

    config = MetricsConfig(temp_config_file, default_enabled=True)

    # Check if the config file was recreated
    assert config.server_uuid is not None
    assert config.did_exist_before is True


def test_default_config_values(temp_config_file):
    """
    Test default config values if a new config file is created.
    """
    config = MetricsConfig(temp_config_file, default_enabled=True)

    assert config.enabled is True
    assert config.log_errors_enabled is False
    assert config.log_sent_data_enabled is False
    assert config.log_response_status_text_enabled is False


def test_config_values_after_updating(temp_config_file):
    """
    Test whether the loaded config values are correct after updating some fields.
    """
    default_enabled = False
    config = MetricsConfig(temp_config_file, default_enabled=default_enabled)

    with open(temp_config_file, "r", encoding="utf-8") as f:
        config_data = tomlkit.load(f)

    config_data["enabled"] = not default_enabled
    config_data["log-errors"] = True
    config_data["log-sent-data"] = True
    config_data["log-response-status-text"] = True

    with open(temp_config_file, "w", encoding="utf-8") as f:
        tomlkit.dump(config_data, f)

    config.load_config()  # Reload the configuration

    assert config.enabled == (not default_enabled)
    assert config.log_errors_enabled is True
    assert config.log_sent_data_enabled is True
    assert config.log_response_status_text_enabled is True



import os
import platform
from unittest.mock import MagicMock

import pytest

from endstone_bstats import Metrics
from endstone_bstats._executor import ScheduledThreadPoolExecutor


@pytest.fixture
def plugin(tmp_path):
    p = MagicMock()
    p.data_folder = str(tmp_path)
    p.server.scheduler.run_task = MagicMock()
    p.enabled = True
    p.logger.warning = MagicMock()
    p.logger.info = MagicMock()
    p.server.online_players = ["Player1", "Player2", "Player3"]
    p.description.version = "1.0.0"
    return p


@pytest.fixture
def metrics(mocker, plugin):
    mocker.patch.object(ScheduledThreadPoolExecutor, "submit")
    mocker.patch.object(ScheduledThreadPoolExecutor, "submit_at_fixed_rate")
    m = Metrics(plugin, service_id=1234)
    yield m
    m.shutdown()


def test_init(metrics, plugin):
    assert metrics._plugin == plugin
    assert metrics._metrics_base is not None


def test_add_custom_chart(mocker, metrics):
    chart = mocker.MagicMock()
    metrics.add_custom_chart(chart)
    assert chart in metrics._metrics_base._custom_charts


def test_append_platform_data(metrics):
    platform_data = {}
    metrics.append_platform_data(platform_data)

    assert platform_data["playerAmount"] == 3
    assert platform_data["osName"] == platform.system()
    assert platform_data["osArch"] == platform.machine().lower()
    assert platform_data["osVersion"] == platform.release()
    assert platform_data["coreCount"] == os.cpu_count()


def test_append_service_data(metrics):
    service_data = {}
    metrics.append_service_data(service_data)
    assert service_data["pluginVersion"] == "1.0.0"



import threading
import time
from datetime import datetime

import pytest

from endstone_bstats._executor import ScheduledThreadPoolExecutor


@pytest.fixture
def executor():
    exec = ScheduledThreadPoolExecutor(max_workers=2)
    yield exec
    exec.shutdown()


def test_initial_submission(executor):
    executed = threading.Event()

    def task():
        executed.set()

    executor.submit(task, delay_secs=1)
    time.sleep(2)
    assert executed.is_set()


def test_fixed_rate_submission(executor):
    execution_times = []

    def task():
        execution_times.append(datetime.now())

    executor.submit_at_fixed_rate(task, initial_delay_secs=1, period_secs=2)

    time.sleep(7)  # Let the task run a few times.
    assert len(execution_times) >= 3


def test_shutdown(executor):
    executed = threading.Event()

    def task():
        executed.set()

    executor.submit(task, delay_secs=2)
    executor.shutdown()
    assert executor.shutdown_event.is_set()
    assert not executed.is_set()


def test_submit_args(executor):
    result = []

    def append_with_args(value):
        result.append(value)

    executor.submit(append_with_args, delay_secs=1, value=42)
    time.sleep(2)
    assert result == [42]


def test_repeated_task_execution(executor):
    lock = threading.Lock()
    count = 0

    def increment():
        nonlocal count
        with lock:
            count += 1

    executor.submit_at_fixed_rate(increment, initial_delay_secs=0, period_secs=1)
    time.sleep(5)

    # At least 4 successful increments are expected because of 4 seconds delay after initial execution.
    with lock:
        assert count >= 4



import pytest

from endstone_bstats import ChartDataError, CustomChart


class TestCustomChart(CustomChart):
    __test__ = False

    def __init__(self, chart_id, data):
        super().__init__(chart_id)
        self._data = data

    def get_chart_data(self):
        if isinstance(self._data, Exception):
            raise self._data

        return self._data


def test_valid_chart_data():
    chart = TestCustomChart("valid_chart", {"key": "value"})
    expected_result = {"chartId": "valid_chart", "data": {"key": "value"}}
    assert chart._get_request_json_object() == expected_result


def test_empty_chart_id():
    with pytest.raises(ValueError, match="chart_id cannot be None or empty!"):
        TestCustomChart("", {"key": "value"})


def test_chart_data_returns_none():
    chart = TestCustomChart("chart_with_none_data", None)
    assert chart._get_request_json_object() is None


def test_chart_data_exception():
    exception_message = "Data retrieval error"
    chart = TestCustomChart("chart_throws_exception", Exception(exception_message))

    with pytest.raises(
        ChartDataError,
        match="Failed to get data for custom chart with id chart_throws_exception",
    ) as exc_info:
        chart._get_request_json_object()

    assert str(exc_info.value.__cause__) == exception_message
